@@ -13,7 +13,7 @@ use MIME::Base64 qw(encode_base64url);
 use WWW::Mechanize;
 use URI;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 my $home_dir;
 
@@ -61,7 +61,7 @@ sub new {
 }
 sub api {
 
-    my ($self, $endpoint_name, $id) = @_;
+    my ($self, $endpoint_name, $id, $api_params) = @_;
 
     if (! defined $endpoint_name) {
         croak "Tesla::API::api() requires an endpoint name sent in";
@@ -74,8 +74,8 @@ sub api {
     my $uri = $endpoint->{URI};
 
     if ($uri =~ /\{/) {
-        if (! defined $id) {
-            croak "Endpoint $endpoint_name requires an \$id, but none sent in";
+        if (! defined $id || $id !~ /^\d+$/) {
+            croak "Endpoint $endpoint_name requires an \$id as an integer";
         }
         $uri =~ s/\{.*?\}/$id/;
     }
@@ -94,17 +94,15 @@ sub api {
 
     my $url = URI->new(API_URL . $uri);
 
-    my $request;
+    my $header = ['Content-Type' => 'application/json; charset=UTF-8'];
 
     if ($auth) {
         my $token_string = "Bearer " . $self->_access_token;
-        my $header = ['Authorization' => $token_string];
-        $request = HTTP::Request->new($type, $url, $header);
-    }
-    else {
-        $request = HTTP::Request->new($type, $url);
+        push @$header, 'Authorization' => $token_string;
     }
 
+    my $request = HTTP::Request->new($type, $url, $header, encode_json($api_params));
+
     my $response = $self->mech->request($request);
 
     if ($response->is_success) {
@@ -122,6 +120,10 @@ sub api {
         warn $response->status_line;
     }
 }
+sub api_cache_clear {
+    my ($self) = @_;
+    %api_cache = ();
+}
 sub api_cache_time {
     my ($self, $cache_seconds) = @_;
 
@@ -450,31 +452,30 @@ Tesla::API - Interface to Tesla's API
 
     my $tesla = Tesla::API->new;
 
-    my $vehicles = $tesla->my_vehicles;
-
-    my $vehicle_name = $tesla->vehicle_name;
-
     my @endpoint_names = keys %{ $tesla->endpoints };
 
-    my $endpoint_name = 'VEHICLE_DATA';
+    # See Tesla::Vehicle for direct access to vehicle-related methods
 
-    my $car_data = $tesla->api($endpoint_name, $tesla->id);
+    my $endpoint_name   = 'VEHICLE_DATA';
+    my $vehicle_id      = 3234234242124;
+
+    my $car_data = $tesla->api($endpoint_name, $vehicle_id);
+
+    if ($tesla->api('ACTUATE_TRUNK', $vehicle_id, {which_trunk => 'rear'})) {
+        put_stuff_in_trunk();
+    }
 
 =head1 DESCRIPTION
 
 This distribution provides access to the Tesla API.
 
-B<WARNING>: This is an initial, beta release. The interface *WILL* change.
+B<WARNING>: This is an initial, beta release. The interface may change.
 
 This class is designed to be subclassed. For example, I have already begun a
 new L<Tesla::Vehicle> distribution which will have access and update methods
 that deal specifically with Tesla autos, then a C<Tesla::Powerwall>
 distribution for their battery storage etc.
 
-Some endpoints require an ID sent in, so it must be provided for those calls
-as well. Some endpoints also require additional parameters for setting
-attributes, but I haven't got that far yet.
-
 =head1 METHODS
 
 =head2 new(%params)
@@ -517,7 +518,7 @@ to Tesla every time.
 
 I<Return>: Integer, the number of seconds we're caching Tesla API data for.
 
-=head2 api($endpoint, $id)
+=head2 api($endpoint, $id, $api_params)
 
 Responsible for disseminating the endpoints and retrieving data through the
 Tesla API.
@@ -526,7 +527,7 @@ B<Parameters>:
 
     $endpoint
 
-Mandatory, String: A valid Tesla API endpoint name. The entire list can be
+I<Mandatory, String>: A valid Tesla API endpoint name. The entire list can be
 found in the C<t/test_data/endpoints.json> file for the time being.
 
     $id
@@ -534,8 +535,29 @@ found in the C<t/test_data/endpoints.json> file for the time being.
 I<Optional, Integer>: Some endpoints require an ID sent in (eg. vehicle ID,
 Powerwall ID etc).
 
+    $api_params
+
+I<Optional, Hash Reference>: Some API calls require additional parameters. Send
+in a hash reference where the keys are the API parameter name, and the value is,
+well, the value.
+
 I<Return>: Hash or array reference, depending on the endpoint.
 
+=head2 api_cache_clear
+
+Some methods chain method calls. For example, calling
+C<< $vehicle->doors_lock >> will poll the API, then cache the state data.
+
+if another call is made to C<< $vehicle->locked >> immediately thereafter to
+check whether the door is actually closed or not, the old cached data would
+normally be returned.
+
+If we don't clear the cache out between these two calls, we will be returned
+stale data.
+
+Takes no parameters, has no return. Only use this call in API calls that
+somehow manipulate the state of the object you're working with.
+
 =head2 api_cache_time($cache_seconds)
 
 The number of seconds we will cache retrieved endpoint data from the Tesla API
